package com.ycwl.basic.biz;

import com.ycwl.basic.mapper.VideoMapper;
import com.ycwl.basic.model.mobile.order.IsBuyRespVO;
import com.ycwl.basic.model.mobile.order.PriceObj;
import com.ycwl.basic.model.pc.face.entity.FaceEntity;
import com.ycwl.basic.model.pc.order.entity.OrderEntity;
import com.ycwl.basic.model.pc.order.entity.OrderItemEntity;
import com.ycwl.basic.model.pc.order.req.OrderUpdateReq;
import com.ycwl.basic.model.pc.scenic.entity.ScenicConfigEntity;
import com.ycwl.basic.model.pc.scenic.entity.ScenicEntity;
import com.ycwl.basic.model.pc.template.resp.TemplateRespVO;
import com.ycwl.basic.model.pc.video.resp.VideoRespVO;
import com.ycwl.basic.repository.FaceRepository;
import com.ycwl.basic.repository.OrderRepository;
import com.ycwl.basic.repository.ScenicRepository;
import com.ycwl.basic.repository.SourceRepository;
import com.ycwl.basic.repository.TemplateRepository;
import com.ycwl.basic.repository.VideoRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Component
public class OrderBiz {

    @Autowired
    private VideoMapper videoMapper;
    @Autowired
    private ScenicRepository scenicRepository;
    @Autowired
    private TemplateRepository templateRepository;
    @Autowired
    private OrderRepository orderRepository;
    @Autowired
    private FaceRepository faceRepository;
    @Autowired
    private SourceRepository sourceRepository;
    @Autowired
    private VideoRepository videoRepository;

    public PriceObj queryPrice(Long scenicId, int goodsType, Long goodsId) {
        PriceObj priceObj = new PriceObj();
        priceObj.setGoodsType(goodsType);
        priceObj.setGoodsId(goodsId);
        ScenicConfigEntity scenicConfig = scenicRepository.getScenicConfig(scenicId);
        if (scenicConfig != null) {
            if (Integer.valueOf(1).equals(scenicConfig.getAllFree())) {
                // 景区全免
                priceObj.setFree(true);
                priceObj.setPrice(BigDecimal.ZERO);
                priceObj.setSlashPrice(BigDecimal.ZERO);
                return priceObj;
            }
        }
        switch (goodsType) {
            case 0: // video
                VideoRespVO video = videoMapper.getById(goodsId);
                if (video == null) {
                    return null;
                }
                TemplateRespVO template = templateRepository.getTemplate(video.getTemplateId());
                if (template == null) {
                    return priceObj;
                }
                priceObj.setPrice(template.getPrice());
                BigDecimal slashPrice = template.getSlashPrice();
                if (slashPrice == null) {
                    priceObj.setSlashPrice(priceObj.getPrice());
                } else {
                    priceObj.setSlashPrice(slashPrice);
                }
                priceObj.setScenicId(video.getScenicId());
                break;
            case 1: // source
                // goodsId 实际上是人脸ID
                FaceEntity _face = faceRepository.getFace(goodsId);
                if (_face == null || _face.getScenicId() == null) {
                    return null;
                }
                ScenicEntity _scenic = scenicRepository.getScenic(_face.getScenicId());
                if (_scenic == null) {
                    return null;
                }
                priceObj.setPrice(_scenic.getPrice());
                priceObj.setSlashPrice(_scenic.getPrice());
                break;
            case 2: // source
                // goodsId 实际上是人脸ID
                FaceEntity __face = faceRepository.getFace(goodsId);
                if (__face == null || __face.getScenicId() == null) {
                    return null;
                }
                ScenicEntity __scenic = scenicRepository.getScenic(__face.getScenicId());
                if (__scenic == null) {
                    return null;
                }
                priceObj.setPrice(__scenic.getPrice());
                priceObj.setSlashPrice(__scenic.getPrice());
                break;
        }
        return priceObj;
    }

    public IsBuyRespVO isBuy(Long userId, Long scenicId, int goodsType, Long goodsId) {
        IsBuyRespVO respVO = new IsBuyRespVO();
        boolean isBuy = orderRepository.checkUserBuyItem(userId, goodsType, goodsId);
        // 免费送逻辑，之前已经赠送了的
        if (!isBuy) {
            switch (goodsType) {
                case 0:
                    isBuy = videoRepository.getUserIsBuy(userId, goodsId);
                    break;
                case 1:
                case 2:
                    isBuy = sourceRepository.getUserIsBuy(userId, goodsType, goodsId);
                    break;
            }
        } else {
            OrderEntity orderEntity = orderRepository.getUserBuyItem(userId, goodsType, goodsId);
            if (orderEntity != null) {
                respVO.setOrderId(orderEntity.getId());
            }
        }
        // 还是没买
        respVO.setBuy(isBuy);
        if (!isBuy) {
            PriceObj priceObj = queryPrice(scenicId, goodsType, goodsId);
            if (priceObj == null) {
                return respVO;
            }
            respVO.setFree(priceObj.isFree());
            respVO.setGoodsType(goodsType);
            respVO.setGoodsId(goodsId);
            respVO.setPrice(priceObj.getPrice());
            respVO.setSlashPrice(priceObj.getSlashPrice());
        }
        return respVO;
    }

    public void paidOrder(Long orderId) {
        orderRepository.clearOrderCache(orderId); // 检查支付呢，还缓存
        OrderEntity order = orderRepository.getOrder(orderId);
        List<OrderItemEntity> orderItems = orderRepository.getOrderItems(orderId);
        OrderEntity orderUpdate = new OrderEntity();
        orderUpdate.setPayAt(new Date());
        orderUpdate.setStatus(1);
        orderRepository.updateOrder(orderId, orderUpdate);
        orderItems.forEach(item -> {
            switch (item.getGoodsType()) {
                case 0: // vlog视频
                    videoRepository.setUserIsBuyItem(order.getMemberId(), item.getGoodsId(), order.getId());
                case 1: // 视频原素材
                case 2: // 照片原素材
                    sourceRepository.setUserIsBuyItem(order.getMemberId(), item.getGoodsType(), item.getGoodsId(), order.getId());
            }
        });
        orderRepository.clearOrderCache(orderId); // 更新完了，清理下
    }

    public void cancelOrder(Long orderId) {
        orderRepository.clearOrderCache(orderId); // 检查支付呢，还缓存
        OrderEntity order = orderRepository.getOrder(orderId);
        List<OrderItemEntity> orderItems = orderRepository.getOrderItems(orderId);
        OrderEntity orderUpdate = new OrderEntity();
        orderUpdate.setCancelAt(new Date());
        orderUpdate.setStatus(9);
        orderRepository.updateOrder(orderId, orderUpdate);
        orderItems.forEach(item -> {
            switch (item.getGoodsType()) {
                case 0: // vlog视频
                    videoRepository.setUserNotBuyItem(order.getMemberId(), item.getGoodsId());
                case 1: // 视频原素材
                case 2: // 照片原素材
                    sourceRepository.setUserNotBuyItem(order.getMemberId(), item.getGoodsType(), item.getGoodsId());
            }
        });
        orderRepository.clearOrderCache(orderId); // 更新完了，清理下
    }

    public void refundOrder(Long orderId) {
        orderRepository.clearOrderCache(orderId);
        OrderEntity order = orderRepository.getOrder(orderId);
        List<OrderItemEntity> orderItems = orderRepository.getOrderItems(orderId);
        OrderEntity orderUpdate = new OrderEntity();
        orderUpdate.setRefundAt(new Date());
        orderUpdate.setStatus(2);
        orderUpdate.setRefundStatus(1);
        orderRepository.updateOrder(orderId, orderUpdate);
        orderItems.forEach(item -> {
            switch (item.getGoodsType()) {
                case 0: // vlog视频
                    videoRepository.setUserNotBuyItem(order.getMemberId(), item.getGoodsId());
                case 1: // 视频原素材
                case 2: // 照片原素材
                    sourceRepository.setUserNotBuyItem(order.getMemberId(), item.getGoodsType(), item.getGoodsId());
            }
        });
        orderRepository.clearOrderCache(orderId); // 更新完了，清理下
    }
}
