package com.ycwl.basic.device.operator;

import com.alibaba.fastjson.JSON;
import com.ycwl.basic.device.entity.common.FileObject;
import com.ycwl.basic.device.entity.wvp_passive.WvpPassiveStorageConfig;
import com.ycwl.basic.storage.utils.StorageUtil;
import com.ycwl.basic.utils.SnowFlakeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

@Slf4j
public class WvpPassiveStorageOperator extends ADeviceStorageOperator {
    public static class Task {
        public Long taskId;
        public Long scenicId;
        public Long deviceId;
        public String deviceNo;
        public Date startTime;
        public Date endTime;
    }

    private static List<Task> taskList = Collections.synchronizedList(new ArrayList<>());
    private static ConcurrentHashMap<Long, FileObject> fileListMap = new ConcurrentHashMap<>();

    private WvpPassiveStorageConfig config;

    public WvpPassiveStorageOperator(String configJson) {
        loadConfig(configJson);
    }

    public static String getUrlForTask(Long taskId) {
        return StorageUtil.joinPath("video-source", taskId.toString() + ".mp4");
    }

    public static void onReceiveResult(Long taskId, FileObject fileObject) {
        if (fileObject == null) {
            log.info("任务{}获取视频失败！", taskId);
            fileListMap.put(taskId, new FileObject());
        } else {
            fileListMap.put(taskId, fileObject);
        }
    }

    public static List<Task> getTaskListByScenicId(Long scenicId) {
        List<Task> scenicTask = taskList.stream().filter(task -> task.scenicId.equals(scenicId)).collect(Collectors.toList());
        if (scenicTask.isEmpty()) {
            return Collections.emptyList();
        }
        taskList.removeAll(scenicTask);
        return scenicTask;
    }

    @Override
    public void loadConfig(String configJson) {
        this.config = JSON.parseObject(configJson, WvpPassiveStorageConfig.class);
    }

    @Override
    public List<FileObject> getFileListByDtRange(Date startDate, Date endDate) {
        Task task = new Task();
        task.taskId = SnowFlakeUtil.getLongId();
        task.scenicId = device.getScenicId();
        task.deviceId = device.getId();
        if (StringUtils.isNotBlank(config.getDeviceNo())) {
            task.deviceNo = config.getDeviceNo();
        } else {
            task.deviceNo = device.getNo2();
        }
        task.startTime = startDate;
        task.endTime = endDate;
        taskList.add(task);
        Date taskStartTime = new Date();
        while (true) {
            if (new Date().getTime() - taskStartTime.getTime() > 80000L) {
                log.info("任务{}获取视频超时！", task.taskId);
                return Collections.emptyList();
            }
            if (fileListMap.containsKey(task.taskId)) {
                FileObject fileObject = fileListMap.get(task.taskId);
                if (StringUtils.isNotBlank(fileObject.getUrl())) {
                    log.info("任务{}获取视频成功！{}", task.taskId, fileObject);
                    fileListMap.remove(task.taskId);
                    return Collections.singletonList(fileObject);
                } else {
                    log.info("任务{}获取视频失败！", task.taskId);
                    return Collections.emptyList();
                }
            } else {
                try {
                    Thread.sleep(1000L);
                } catch (InterruptedException e) {
                    return Collections.emptyList();
                }
            }
        }
    }

    @Override
    public boolean removeFilesBeforeDate(Date date) {
        // 不做删除
        return false;
    }
}
