package com.ycwl.basic.service.impl.pc;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ycwl.basic.biz.OrderBiz;
import com.ycwl.basic.enums.StatisticEnum;
import com.ycwl.basic.mapper.FaceSampleMapper;
import com.ycwl.basic.mapper.SourceMapper;
import com.ycwl.basic.mapper.StatisticsMapper;
import com.ycwl.basic.mapper.FaceMapper;
import com.ycwl.basic.model.jwt.JwtInfo;
import com.ycwl.basic.model.mobile.face.FaceRecognizeResp;
import com.ycwl.basic.model.mobile.order.IsBuyRespVO;
import com.ycwl.basic.model.mobile.statistic.req.StatisticsRecordAddReq;
import com.ycwl.basic.model.pc.face.entity.FaceEntity;
import com.ycwl.basic.model.pc.face.req.FaceReqQuery;
import com.ycwl.basic.model.pc.face.resp.FaceRespVO;
import com.ycwl.basic.model.pc.source.entity.MemberSourceEntity;
import com.ycwl.basic.model.pc.source.entity.SourceEntity;
import com.ycwl.basic.model.task.resp.SearchFaceRespVo;
import com.ycwl.basic.repository.FaceRepository;
import com.ycwl.basic.service.pc.FaceService;
import com.ycwl.basic.service.task.TaskFaceService;
import com.ycwl.basic.service.task.TaskService;
import com.ycwl.basic.storage.StorageFactory;
import com.ycwl.basic.storage.adapters.IStorageAdapter;
import com.ycwl.basic.storage.utils.StorageUtil;
import com.ycwl.basic.task.VideoPieceGetter;
import com.ycwl.basic.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

import static com.ycwl.basic.constant.FaceConstant.USER_FACE_DB_NAME;

/**
 * @Author：longbinbin
 * @Date：2024/12/2 16:39
 */
@Slf4j
@Service
public class FaceServiceImpl implements FaceService {
    @Autowired
    private FaceMapper faceMapper;
    @Autowired
    private TaskFaceService faceService;
    @Autowired
    private StatisticsMapper statisticsMapper;

    @Autowired
    private TaskService taskTaskService;
    @Autowired
    private SourceMapper sourceMapper;
    @Autowired
    private FaceSampleMapper faceSampleMapper;
    @Autowired
    private OrderBiz orderBiz;
    @Autowired
    private FaceRepository faceRepository;

    @Override
    public ApiResponse<PageInfo<FaceRespVO>> pageQuery(FaceReqQuery faceReqQuery) {
        PageHelper.startPage(faceReqQuery.getPageNum(),faceReqQuery.getPageSize());
        List<FaceRespVO> list = faceMapper.list(faceReqQuery);
        PageInfo<FaceRespVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    public ApiResponse<List<FaceRespVO>> list(FaceReqQuery faceReqQuery) {
        return ApiResponse.success(faceMapper.list(faceReqQuery));
    }

    @Override
    public ApiResponse<FaceRespVO> getById(Long id) {
        return ApiResponse.success(faceMapper.getById(id));
    }

    @Override
    public ApiResponse<Integer> add(FaceEntity face) {
        face.setId(SnowFlakeUtil.getLongId());
        int i = faceMapper.add(face);
        if (i == 0) {
            return ApiResponse.fail("添加失败");
        }
        return ApiResponse.success(i);
    }

    @Override
    public ApiResponse<Integer> deleteById(Long id) {
        int i = faceMapper.deleteById(id);
        if (i == 0) {
            return ApiResponse.fail("删除失败");
        }
        return ApiResponse.success(i);
    }

    @Override
    public ApiResponse<Integer> deleteByIds(List<Long> ids) {
        int i = faceMapper.deleteByIds(ids);
        if (i == 0) {
            return ApiResponse.fail("删除失败");
        }
        return ApiResponse.success(i);
    }

    @Override
//    @Transactional(rollbackFor = Exception.class)
    public ApiResponse faceUpload(MultipartFile file, Long scenicId) {
        //获取用户id
        JwtInfo worker = JwtTokenUtil.getWorker();
        Long userId = worker.getUserId();
        log.info("当前登录用户信息：{}",worker);

        //1、上传人脸照片
        IStorageAdapter adapter = StorageFactory.use("faces");
        String filePath = StorageUtil.joinPath("user-faces", DateUtils.format(new Date(),"yyyy-MM-dd"));
        String originalFilename = file.getOriginalFilename();
        String suffix = originalFilename.split("\\.", 2)[1];
        String fileName = UUID.randomUUID().toString() + "." + suffix;
        String faceUrl = adapter.uploadFile(file, filePath, fileName);
        Long newFaceId = SnowFlakeUtil.getLongId();
        Long oldFaceId = null;
        SearchFaceRespVo userDbSearchResult = faceService.searchFace(USER_FACE_DB_NAME+scenicId, faceUrl);
        float strictScore = 0.6F;
        if (userDbSearchResult == null) {
            // 都是null了，那得是新的
            faceService.addFaceSample(USER_FACE_DB_NAME+scenicId, newFaceId.toString(), faceUrl, newFaceId.toString());
        } else if (userDbSearchResult.getSampleListIds() == null || userDbSearchResult.getSampleListIds().isEmpty()) {
            // 没有匹配到过，也得是新的
            faceService.addFaceSample(USER_FACE_DB_NAME+scenicId, newFaceId.toString(), faceUrl, newFaceId.toString());
        } else if (userDbSearchResult.getFirstMatchRate() < strictScore) {
            // 有匹配结果，但是不匹配旧的
            faceService.addFaceSample(USER_FACE_DB_NAME+scenicId, newFaceId.toString(), faceUrl, newFaceId.toString());
        } else {
            // 有匹配结果，且能匹配旧的数据
            Optional<Long> faceAny = userDbSearchResult.getSampleListIds().stream().filter(_faceId -> {
                FaceEntity face = faceRepository.getFace(_faceId);
                if (face == null) {
                    return false;
                }
                return face.getMemberId().equals(userId);
            }).findAny();
            if (faceAny.isPresent()) {
                oldFaceId = faceAny.get();
                FaceRespVO oldFace = faceMapper.getById(oldFaceId);
                if (oldFace == null) {
                    faceService.deleteFaceSample(USER_FACE_DB_NAME+scenicId, oldFaceId.toString());
                    faceService.addFaceSample(USER_FACE_DB_NAME+scenicId, newFaceId.toString(), faceUrl, newFaceId.toString());
                    oldFaceId = null;
                } else {
                    newFaceId = oldFaceId;
                }
            }
        }
        if (oldFaceId==null) {
            //新增人脸
            FaceEntity faceEntity = new FaceEntity();
            faceEntity.setCreateAt(new Date());
            faceEntity.setScenicId(scenicId);
            faceEntity.setMemberId(userId);
            faceEntity.setFaceUrl(faceUrl);
            faceEntity.setId(newFaceId);
            faceMapper.add(faceEntity);
        } else {
            //更新人脸
            FaceEntity faceEntity = new FaceEntity();
            faceEntity.setId(oldFaceId);
            faceEntity.setFaceUrl(faceUrl);
            faceMapper.update(faceEntity);
            faceRepository.clearFaceCache(oldFaceId);
        }
        StatisticsRecordAddReq statisticsRecordAddReq = new StatisticsRecordAddReq();
        statisticsRecordAddReq.setMemberId(userId);
        statisticsRecordAddReq.setType(StatisticEnum.UPLOAD_FACE.code);
        statisticsRecordAddReq.setScenicId(scenicId);
        statisticsRecordAddReq.setMorphId(newFaceId);
        statisticsMapper.addStatisticsRecord(statisticsRecordAddReq);
        FaceRecognizeResp resp = new FaceRecognizeResp();
        resp.setUrl(faceUrl);
        resp.setFaceId(newFaceId);
        matchFaceId(newFaceId);
        return ApiResponse.success(resp);
    }

    @Override
    public List<FaceRespVO> listByUser(Long userId, String scenicId) {
        return faceMapper.listByScenicAndUserId(scenicId, userId);
    }

    @Override
    public SearchFaceRespVo matchFaceId(Long faceId) {
        FaceEntity face = faceRepository.getFace(faceId);
        if (face == null) {
            return null;
        }
        SearchFaceRespVo scenicDbSearchResult = faceService.searchFace(face.getScenicId(), face.getFaceUrl());
        FaceEntity faceEntity = new FaceEntity();
        faceEntity.setId(faceId);
        faceEntity.setScore(scenicDbSearchResult.getScore());
        faceEntity.setMatchResult(scenicDbSearchResult.getSearchResultJson());
        if (scenicDbSearchResult.getFirstMatchRate() != null) {
            faceEntity.setFirstMatchRate(BigDecimal.valueOf(scenicDbSearchResult.getFirstMatchRate()));
        }
        if (scenicDbSearchResult.getSampleListIds() != null) {
            faceEntity.setMatchSampleIds(scenicDbSearchResult.getSampleListIds().stream().map(String::valueOf).collect(Collectors.joining(",")));
        }
        faceEntity.setCreateAt(new Date());
        faceEntity.setScenicId(face.getScenicId());
        faceEntity.setMemberId(face.getMemberId());
        faceEntity.setFaceUrl(face.getFaceUrl());
        List<Long> sampleListIds = scenicDbSearchResult.getSampleListIds();
        faceMapper.update(faceEntity);
        faceRepository.clearFaceCache(faceEntity.getId());
        if (sampleListIds != null && !sampleListIds.isEmpty()) {// 匹配原片：照片
            List<SourceEntity> sourceEntities = sourceMapper.listBySampleIds(sampleListIds);
            List<MemberSourceEntity> memberSourceEntityList = sourceEntities.stream().map(sourceEntity -> {
                MemberSourceEntity memberSourceEntity = new MemberSourceEntity();
                memberSourceEntity.setScenicId(face.getScenicId());
                memberSourceEntity.setFaceId(face.getId());
                memberSourceEntity.setMemberId(face.getMemberId());
                memberSourceEntity.setSourceId(sourceEntity.getId());
                memberSourceEntity.setType(sourceEntity.getType());
                IsBuyRespVO isBuy = orderBiz.isBuy(face.getMemberId(), face.getScenicId(), sourceEntity.getType(), faceEntity.getId());
                if (isBuy.isBuy()) { // 如果用户买过
                    memberSourceEntity.setIsBuy(1);
                } else if (isBuy.isFree()) { // 全免费逻辑
                    memberSourceEntity.setIsBuy(1);
                } else {
                    memberSourceEntity.setIsBuy(0);
                }
                return memberSourceEntity;
            }).collect(Collectors.toList());
            if (!memberSourceEntityList.isEmpty()) {
                sourceMapper.addRelations(memberSourceEntityList);
                taskTaskService.autoCreateTaskByFaceId(face.getId());
//                VideoPieceGetter.Task task = new VideoPieceGetter.Task();
//                task.faceId = face.getId();
//                task.faceSampleIds = sampleListIds;
//                task.memberId = face.getMemberId();
//                VideoPieceGetter.addTask(task);
            }
        }
        return scenicDbSearchResult;
    }

}
